#include <map>
#include <vector>

#include <cstring>

using namespace std;

#include "cache.h"

ReadCache::ReadCache(int cache_entries, INT block_size)
{
    this->cache_entries = cache_entries;
    this->block_size = block_size;
    this->counter = 0;
    this->cache = new RCE[cache_entries];
    for(int i = 0; i < cache_entries; i++)
    {
        this->cache[i].ru = 0;
        this->cache[i].ptr = -1;
        this->cache[i].buf = new char[block_size];
    }
}

ReadCache::~ReadCache()
{
    for(int i = 0; i < cache_entries; i++)
    {
        delete[] cache[i].buf;
    }
    delete[] cache;
}


bool ReadCache::Get(LONGINT ptr, void* buf)
{
    for(int i = 0; i < cache_entries; i++)
    {
        if(cache[i].ptr == ptr)
        {
            counter++;
            cache[i].ru = counter;
            memcpy(buf, cache[i].buf, block_size);
            return true;
        }
    }
    return false;
}

void ReadCache::Insert(LONGINT ptr, void* buf)
{
    if(cache_entries <= 0) return;
    // find oldest cache entry
    LONGINT oldru = counter + 1;
    int oldix;
    for(int i = 0; i < cache_entries; i++)
    {
        if(cache[i].ru < oldru)
        {
            oldru = cache[i].ru;
            oldix = i;
        }
    }
    if(oldix < 0 || oldix >= cache_entries) throw "Failed to locate cache entry";
    // replace oldest cache entry
    counter++;
    cache[oldix].ru = counter;
    cache[oldix].ptr = ptr;
    memcpy(cache[oldix].buf, buf, block_size);
}

WriteCache::WriteCache(INT block_size)
{
    this->block_size = block_size;
}

WriteCache::~WriteCache()
{
    // deallokate all WCE's
    map<int, map<LONGINT, WCE*> >::iterator m = cache.begin();
    while(m != cache.end())
    {
        map<LONGINT, WCE*>::iterator ce = m->second.begin();
        while(ce != m->second.end())
        {
            delete ce->second;
            ce++;
        }
        m++;
    }
}

bool WriteCache::Get(int lvl, LONGINT ptr, void* buf)
{
    map<int, map<LONGINT, WCE*> >::iterator m = cache.find(lvl);
    if(m == cache.end()) return false;
    map<LONGINT, WCE*>::iterator ce = m->second.find(ptr);
    if(ce == m->second.end()) return false;
    ce->second->Get(buf);
    return true;
}

void WriteCache::Insert(int lvl, LONGINT ptr, void* buf)
{
    cache[lvl][ptr] = new WCE(buf, block_size);
}

void WriteCache::GetAll(int lvl, vector<LONGINT>& all)
{
    map<int, map<LONGINT, WCE*> >::iterator m = cache.find(lvl);
    if(m == cache.end()) return;
    map<LONGINT, WCE*>::iterator ce = m->second.begin();
    while(ce != m->second.end())
    {
        all.push_back(ce->first);
        ce++;
    }
}
