#ifndef CACHE_H
#define CACHE_H

#include <map>
#include <vector>

#include <cstring>

using namespace std;

#include "common.h"

// Read Cache Entry
// used by ReadCache class
struct RCE
{
    LONGINT ru;
    LONGINT ptr;
    char* buf;
};

// Write Cache Entry
// used by WriteCache class
class WCE
{
private:
    INT buf_size;
    char* buf;
public:
    //WCE() { }
    WCE(void* buf, INT buf_size) { this->buf_size = buf_size; this->buf = new char[buf_size]; memcpy(this->buf, buf, buf_size); }
    WCE(const WCE& o) { buf_size = o.buf_size; this->buf = new char[buf_size]; memcpy(buf, o.buf, buf_size); }
    WCE& operator=(const WCE& o)  { buf_size = o.buf_size; this->buf = new char[buf_size]; memcpy(buf, o.buf, buf_size); return *this; }
    virtual ~WCE() { delete[] buf; }
    void Get(void* buf) { memcpy(buf, this->buf, buf_size); }
};

// read cache
// used by LogDBFile class
class ReadCache
{
private:
    int cache_entries;
    INT block_size;
    LONGINT counter;
    RCE* cache;
public:
    ReadCache(int cache_entries, INT block_size);
    virtual ~ReadCache();
    bool Get(LONGINT ptr, void* buf);
    void Insert(LONGINT ptr, void* buf);
};

// write cache
// used by LogDB class
class WriteCache
{
private:
    INT block_size;
    map<int, map<LONGINT, WCE*> > cache;
public:
    WriteCache(INT block_size);
    virtual ~WriteCache();
    bool Get(int lvl, LONGINT ptr, void* buf);
    void Insert(int lvl, LONGINT ptr, void* buf);
    void GetAll(int lvl, vector<LONGINT>& all);
};

#endif
