#ifndef LOGDB_H
#define LOGDB_H

#include <iostream>

#include <ctime>

using namespace std;

#include "common.h"
#include "cache.h"
#include "logdbfile.h"

const LONGINT UNUSED = 0x8000000000000000L;

// log database (actually log fixed size persisted array)
// used by client code
class LogDB
{
private:
    INT data_size;
    INT block_size;
    INT data_per_block;
    INT ptr_per_block;
    LONGINT total_data;
    LONGINT total_data_blocks;
    INT levels;
    INT *steps;
    LogDBFile* dbf;
    WriteCache* cache;
    void WriteIndex(LONGINT start_ptr, LONGINT num_ptr);
    void DumpIndex(LONGINT ptr, DUMP_DATA_F dump_data, INT levels_left, ostream& os);
    void DumpData(LONGINT ptr, DUMP_DATA_F dump_data, ostream& os);
    void Load(LONGINT elm, time_t t, LONGINT* ptrs, void* buf, int* ix_offset, int *data_offset); 
    void BatchLoad(LONGINT elm, time_t t, LONGINT* ptrs, void* buf, int* ix_offset, int *data_offset, LONGINT* old_ix_ptr, LONGINT *old_data_ptr); 
public:
    LogDB(LONGINT total_data, INT data_size, INT data_per_block);
    virtual ~LogDB();
    void Init(const char* db_file_name, const char* ix_file_name, GET_DATA_F get_data, int cache_entries = 0);
    void Open(const char* db_file_name, const char* ix_file_name, int cache_entries = 0);
    LONGINT Size();
    void Dump(DUMP_DATA_F dump_data, ostream& os = cout);
    void Dump(DUMP_DATA_F dump_data, time_t t, ostream& os = cout);
    void Read(LONGINT elm, void* res);
    void Read(LONGINT elm, void* res, time_t t);
    void Update(LONGINT elm, void* res);
    void BatchUpdateBegin();
    void BatchUpdate(LONGINT elm, void* res);
    void BatchUpdateCommit();
};

#endif
