#include <cstdio>

using namespace std;

#include "common.h"
#include "logdbfile.h"

void LogDBFile::LoadTS()
{
    FILE *fpts = fopen(ixfnm, "rb");
    if(fpts == NULL) throw "Error opening index file";
    while(!feof(fpts))
    {
        TS one;
        fread(&one, sizeof(TS), 1, fpts);
        ts.push_back(one);
    }
    fclose(fpts);
}

LogDBFile::LogDBFile(const char* db_file_name, const char* ix_file_name, const char* mode, INT block_size, int cache_entries)
{
    strncpy(ixfnm, ix_file_name, sizeof(ixfnm));
    this->fp = fopen(db_file_name, mode);
    if(this->fp == NULL) throw "Error accessing log database file";
    this->block_size = block_size;
    if(mode[0] == 'w')
    {
        FILE* fpts = fopen(ixfnm, "wb");
        if(fpts == NULL) throw "Error creating index file";
        fclose(fpts);
        this->write_ptr = 0;
    }
    else
    {
        LoadTS();
        this->write_ptr = ts.back().root + 1;
    }
    cache = new ReadCache(cache_entries, block_size);
}

LogDBFile::~LogDBFile() 
{
    delete cache;
    fclose(fp);
}

void LogDBFile::Write(void* buf)
{
    fseek(fp, write_ptr * block_size, SEEK_SET);
    fwrite(buf, block_size, 1, fp);
    cache->Insert(write_ptr, buf);
    write_ptr++;
}

void LogDBFile::Write(void* buf, int noblocks)
{
    for(int i = 0; i < noblocks; i++)
    {
        Write((char*)buf + i * block_size);
    }
}

void LogDBFile::Read(LONGINT ptr, void* buf)
{
    if(ptr < 0 || ptr >= write_ptr) throw "Read with invalid pointer attempted";
    if(!cache->Get(ptr, buf)) {
        fseek(fp, ptr * block_size, SEEK_SET);
        fread(buf, block_size, 1, fp);
        cache->Insert(ptr, buf);
    }
}

void LogDBFile::Mark()
{
    TS add;
    add.t = time(NULL);
    add.root = GetLastWritePtr();
    FILE* fpts = fopen(ixfnm, "ab");
    fwrite(&add, sizeof(TS), 1, fpts);
    fclose(fpts);
    ts.push_back(add);
    last = add;
}

LONGINT LogDBFile::GetRoot()
{
    return GetRoot(NOW);
}

LONGINT LogDBFile::GetRoot(time_t t)
{
    if(t == NOW) return GetLastWritePtr();
    if(t == last.t) return last.root;
    for(int i = ts.size() - 1; i >= 0; i--)
    {
        if(ts[i].t <= t)
        {
            last.t = t;
            last.root = ts[i].root;
            return ts[i].root;
        }
    }
    throw "Data requested for time earlier than init";
}

LONGINT LogDBFile::GetLastWritePtr()
{
    return write_ptr - 1;
}
