#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <ctime>

#ifdef _WIN32
extern "C" 
{
void __stdcall Sleep(long int ms);
}
#define sleep(n) Sleep(1000*n)
#else
#include <unistd.h>
#endif

#include "logdb.h"

struct Rec
{
    int iv;
    double xv;
    char sv[19];
};

void get(INT ix, void* buf)
{
    Rec r;
    r.iv = ix;
    r.xv = ix / 100.0;
    sprintf(r.sv, "Record #%d", (int)ix);
    memcpy(buf, &r, sizeof(Rec));
}

char res[100];

char* dump(void* buf)
{
    Rec* r = (Rec*)buf;
    sprintf(res, "%d %f %s", r->iv, r->xv, r->sv);
    return res;
}

void test1()
{
    LogDB db(100, sizeof(Rec), 2);
    db.Init("test1.logdb", "test1.ix", get);
    db.Dump(dump);
}

void test2()
{
    LogDB db(100, sizeof(Rec), 2);
    db.Open("test1.logdb", "test1.ix");
    db.Dump(dump);
}

void test3()
{
    LogDB db(100, sizeof(Rec), 2);
    db.Open("test1.logdb", "test1.ix");
    Rec r;
    for(int i = 0; i < 100; i++)
    {
        db.Read(i, &r);
        cout << dump(&r) << endl;
    }
}

void test4()
{
    LogDB db(100, sizeof(Rec), 2);
    db.Open("test1.logdb", "test1.ix");
    sleep(3);
    Rec r;
    db.Read(5, &r);
    r.iv = r.iv + 1;
    r.xv = r.xv + 1;
    strcat(r.sv, "!!!!");
    db.Update(5, &r);
    db.Dump(dump);
    db.Read(5, &r);
    cout << dump(&r) << endl;
    db.Read(5, &r, time(NULL) - 2);
    cout << dump(&r) << endl;
}

void test5()
{
    LogDB db(100, sizeof(Rec), 2);
    db.Open("test1.logdb", "test1.ix");
    sleep(3);
    Rec r6;
    db.Read(6, &r6);
    r6.iv = r6.iv + 1;
    r6.xv = r6.xv + 1;
    strcat(r6.sv, "!!!!");
    Rec r7;
    db.Read(7, &r7);
    r7.iv = r7.iv + 1;
    r7.xv = r7.xv + 1;
    strcat(r7.sv, "!!!!");
    db.BatchUpdateBegin();
    db.BatchUpdate(6, &r6);
    db.BatchUpdate(7, &r7);
    db.BatchUpdateCommit();
    db.Dump(dump);
    db.Read(6, &r6);
    cout << dump(&r6) << endl;
    db.Read(6, &r6, time(NULL) - 2);
    cout << dump(&r6) << endl;
}

int main()
{
    test1();
    test2();
    test3();
    test4();
    test5();
    return EXIT_SUCCESS;
}
