#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#ifdef _WIN32
void __stdcall Sleep(long int ms);
#define sleep(n) Sleep(1000*n)
#else
#include <unistd.h>
#endif

#include "capi.h"

struct Rec
{
    int iv;
    double xv;
    char sv[19];
};

typedef struct Rec Rec;

void get(INT ix, void* buf)
{
    Rec r;
    r.iv = ix;
    r.xv = ix / 100.0;
    sprintf(r.sv, "Record #%d", (int)ix);
    memcpy(buf, &r, sizeof(Rec));
}

char res[100];

char* dump(void* buf)
{
    Rec* r = (Rec*)buf;
    sprintf(res, "%d %f %s", r->iv, r->xv, r->sv);
    return res;
}

void test1()
{
    LOGDB_CTX db;
    logdb_allocate(&db, 100, sizeof(Rec), 2);
    logdb_init(&db, "test3.logdb", "test3.ix", get, 0);
    logdb_free(&db);
}

void test2()
{
    LOGDB_CTX db;
    logdb_allocate(&db, 100, sizeof(Rec), 2);
    logdb_open(&db, "test3.logdb", "test3.ix", 0);
    logdb_free(&db);
}

void test3()
{
    LOGDB_CTX db;
    Rec r;
    int i;
    logdb_allocate(&db, 100, sizeof(Rec), 2);
    logdb_open(&db, "test3.logdb", "test3.ix", 0);
    for(i = 0; i < 100; i++)
    {
        logdb_read(&db, i, &r);
        printf("%s\n", dump(&r));
    }
    logdb_free(&db);
}

void test4()
{
    LOGDB_CTX db;
    Rec r;
    logdb_allocate(&db, 100, sizeof(Rec), 2);
    logdb_open(&db, "test3.logdb", "test3.ix", 0);
    sleep(3);
    logdb_read(&db, 5, &r);
    r.iv = r.iv + 1;
    r.xv = r.xv + 1;
    strcat(r.sv, "!!!!");
    logdb_update(&db, 5, &r);
    logdb_read(&db, 5, &r);
    printf("%s\n", dump(&r));
    logdb_read_historic(&db, 5, &r, time(NULL) - 2);
    printf("%s\n", dump(&r));
    logdb_free(&db);
}

void test5()
{
    LOGDB_CTX db;
    Rec r6;
    Rec r7;
    logdb_allocate(&db, 100, sizeof(Rec), 2);
    logdb_open(&db, "test3.logdb", "test3.ix", 0);
    sleep(3);
    logdb_read(&db, 6, &r6);
    r6.iv = r6.iv + 1;
    r6.xv = r6.xv + 1;
    strcat(r6.sv, "!!!!");
    logdb_read(&db, 7, &r7);
    r7.iv = r7.iv + 1;
    r7.xv = r7.xv + 1;
    strcat(r7.sv, "!!!!");
    logdb_batch_update_begin(&db);
    logdb_batch_update(&db, 6, &r6);
    logdb_batch_update(&db, 7, &r7);
    logdb_batch_update_commit(&db);
    logdb_read(&db, 6, &r6);
    printf("%s\n", dump(&r6));
    logdb_read_historic(&db, 6, &r6, time(NULL) - 2);
    printf("%s\n", dump(&r6));
    logdb_free(&db);
}

int main()
{
    test1();
    test2();
    test3();
    test4();
    test5();
    return EXIT_SUCCESS;
}
